/*
*   
*   
*   This file is part of DSFML
*
*   This software is provided 'as-is', without any express or
*   implied warranty. In no event will the authors be held
*   liable for any damages arising from the use of this software.
*
*   Permission is granted to anyone to use this software for any purpose,
*   including commercial applications, and to alter it and redistribute
*   it freely, subject to the following restrictions:
*
*   1.  The origin of this software must not be misrepresented;
*       you must not claim that you wrote the original software.
*       If you use this software in a product, an acknowledgment
*       in the product documentation would be appreciated but
*       is not required.
*
*   2.  Altered source versions must be plainly marked as such,
*       and must not be misrepresented as being the original software.
*
*   3.  This notice may not be removed or altered from any
*       source distribution.
*/

module dsfml.audio.soundrecorder;

import dsfml.audio.external;
import dsfml.audio.soundbuffer;

import dsfml.system.alloc;
import dsfml.system.pointerchecker;
import dsfml.system.sleep;
import dsfml.system.linkedlist;
import dsfml.system.mutex;
import dsfml.system.lock;
import dsfml.system.thread;


/**
*   SoundRecorder is an interface for capturing sound data.
*   
*   $(B Callback or processSamples override will be called by a different thread, take care of synchronization issues.)
*   
*   Examples:
*   
*   Can be used directly :
*   -------
*   bool callback(short[] data, void* userData)
*   {
*       //Process data
*       
*       return true; //return true to continue capture, else return false  
*   }
*
*   class MyClass
*   {
*       public bool myCallback(short[] data, void* userData)
*      {
*           //Process data
*      
*           return true;
*       }
*   }
*
*   void main()
*   {
*       SoundRecorder record = new SoundRecorder(&callback);
*       // or ...
*       MyClass myClass = new MyClass();
*       SoundRecorder record = new SoundRecorder(&myClass.myCallback);
*       
*
*       record.Start(); //Start recording with default sample rate.
*   }
*   -------
*   
*   or as base class
*   
*   -------
*   
*   class MySoundRecorder : SoundRecorder
*   {
*       this()
*       {
*             
*       }
*       
*       //Override this method
*       protected bool processSamples(out short[])
*       {
*           // Process data here
*           
*           return true; //return true to continue capture, else return false  
*       }
*   } 
*   -------
*/
class SoundRecorder
{
    alias bool function(short[], void*) FuncType;
    alias bool delegate(short[], void*) DelegType;
    
    /**
    *   Construct the sound recorder with a callback function for processing captured samples
    *
    *   Params: 
    *       callback = Callback for processing captured samples
    *       userData = Data to pass to the callback function (null by default)
    */
    this(bool function(short[], void*) callback, void* userData = null)
    {
        m_fcallback = callback;
        m_userData = userData;
        m_id = ++seed;
        m_type = Type.FUNCTION;
        
        init();
    }

    /**
    *   Construct the sound recorder with a delegate for processing captured samples
    *
    *   Params: 
    *       callback = Callback for processing captured samples
    *       userData = Data to pass to the callback function (null by default)
    */
    this(bool delegate(short[], void*) callback, void* userData = null)
    {
        m_dcallback = callback;
        m_userData = userData;
        m_id = ++seed;
        m_type = Type.DELEGATE;
        
        init();        
    }

    /**
    *   Destructor
    */
    ~this()
    {
        if (!m_disposed)
        {
            if (m_flag)
            {
                stop();
            }
            m_instances.remove(m_id);
            sfSoundRecorder_Destroy(m_SoundRecorder);
        }
    }


    /**
    *   Start the capture.
    *   
    *   Only one capture can happen at the same time
    *
    *   Params: 
    *       sampleRate : Sound frequency (the more samples, the higher the quality)
    *                    (44100 by default = CD quality)
    */
    void start(uint sampleRate = 44100)
    {
        sfSoundRecorder_Start(m_SoundRecorder, sampleRate);
        m_t = new Thread(&threadPoll);
        m_t.launch();
    }

    /**
    *   Stop the capture
    */
    void stop()
    {
        sfSoundRecorder_Stop(m_SoundRecorder);
        m_flag = false;
        m_t.wait();
        m_t = null;
    }

    /**
    *   Get the sample rate
    *
    *   Returns: 
    *       Frequency, in samples per second
    */
    uint getSampleRate() 
    {
        if (m_SoundRecorder !is null)
            return sfSoundRecorder_GetSampleRate(m_SoundRecorder);
        else return 0;
    }

    /**
    *   Tell if the system supports sound capture.
    *   If not, this class won't be usable
    *
    *   Returns:
    *       True if audio capture is supported
    *
    */
    static bool canCapture()
    {
        return cast(bool)sfSoundRecorder_CanCapture();
    }

protected:
    /**
    *   Protected constructor
    */   
    this()
    {
        m_type = Type.BASE;
        
        init();
    }

    /**
    *   callback function
    */        
    bool processSamples(short[] s)
    {
        return true;
    }

    sfSoundRecorder* m_SoundRecorder;
    bool m_disposed;
    
private:
    /*
    *   an init function to initialize id of the object.
    */    
    void init()
    {
        // if pointer has not been already created
        if (m_SoundRecorder is null)
        {
            m_id = ++seed;
            m_instances[m_id] = this;
            m_SoundRecorder = sfSoundRecorder_Create(&internalCallback, &m_id);
            
            m_list = new LinkedList!(Samples)();
            
            m_flag = true;
            m_continue = true;
            
            m_mutex = new Mutex();
        }
    }
    
    union
    {
        FuncType m_fcallback;
        DelegType m_dcallback;
    }
    
    void* m_userData;
    int m_id;
    
    static int seed = 0;
    static SoundRecorder[int] m_instances;
    
    enum Type
    {
        BASE,
        DELEGATE,
        FUNCTION
    }

    Type m_type;

    /*
    *   Extern C callback function
    *           
    *   This function must be static for C interop. To retrieve the current
    *   instance, we retrieve id of the sender in the user data, and search associated instance
    *   in the associative array.
    *   
    *   We don't call delegate or derived class on that thread because GC is not aware of this thread
    *   instead we enqueue data informations in a queue and poll this queue with a managed thread.                                  
    */
    extern(C) static int internalCallback(short* s, size_t size, void* user)
    {
        int id;
        // retrieve instance
        if ((id = *cast(int*)(user)) in m_instances)
        {
            SoundRecorder temp = m_instances[id];
            scope Lock l = new Lock(temp.m_mutex);
            if (temp.m_continue)
                // this new is allowed because Samples is an custom alloc class.
                temp.m_list.enqueue(new Samples(s, size));
            return temp.m_continue;
        }
        return false; 
    }
    
    /*
    *   Managed thread loop
    */    
    void threadPoll(void* user)
    {
        while (m_flag)
        {
            sleep(0.05f);
            // if samples are available
            if (!m_list.empty)
            {
                // Lock ressources
                scope Lock l = new Lock(m_mutex);
                
                Samples s = m_list.dequeue;
                //If used directly
                if (this.m_type == Type.FUNCTION)
                    m_continue = this.m_fcallback(s.data[0..s.length].dup, m_userData);
                else if (this.m_type == Type.DELEGATE)
                    m_continue = this.m_dcallback(s.data[0..s.length].dup, m_userData);
                //else this is a base class
                else
                    m_continue = this.processSamples(s.data[0..s.length].dup);
                
                delete s;
                
                if (!m_continue)
                {
                    // delete all samples left
                    foreach(Samples dummy; m_list)
                        delete dummy;
                    break;
                }
            }
        }
    }
    
    Mutex m_mutex;
    
    bool m_flag;
    bool m_continue = true;
    LinkedList!(Samples) m_list;
    Thread m_t;
    
    mixin TEnforcePointer!(m_SoundRecorder);
    // Encapsulation of samples pointers and size


}

// Use explicit alloc to allow instaciation by C thread
private class Samples
{
    mixin Alloc;
    
    this(short* data, size_t length)
    {
        this.data = data;
        this.length = length;
    }

    public short* data;
    public size_t length;
}

