/*
*   This file is part of DSFML
*
*   This software is provided 'as-is', without any express or
*   implied warranty. In no event will the authors be held
*   liable for any damages arising from the use of this software.
*
*   Permission is granted to anyone to use this software for any purpose,
*   including commercial applications, and to alter it and redistribute
*   it freely, subject to the following restrictions:
*
*   1.  The origin of this software must not be misrepresented;
*       you must not claim that you wrote the original software.
*       If you use this software in a product, an acknowledgment
*       in the product documentation would be appreciated but
*       is not required.
*
*   2.  Altered source versions must be plainly marked as such,
*       and must not be misrepresented as being the original software.
*
*   3.  This notice may not be removed or altered from any
*       source distribution.
*/

module dsfml.audio.soundstream;

import dsfml.system.alloc;
import dsfml.system.vector3;
import dsfml.system.linkedlist;
import dsfml.system.lock;
import dsfml.system.mutex;
import dsfml.system.sleep;
import dsfml.system.thread;

import dsfml.audio.sound;
import dsfml.audio.soundstatus;
import dsfml.audio.external;

/**
*   SoundStream is a streamed sound, ie samples are acquired
*   while the sound is playing. Use it for big sounds that would
*   require hundreds of MB in memory, or for streaming sound from the network.
*   
*   SoundStream is a base class and cannot be instanciated directly.
*   
*   $(B onGetData override will be called by a different thread, take care of synchronization issues.) onStart is called by the thread which called .play().
*   
*   ------------------------
*   class MySoundStream : SoundStream
*   {
*       this()
*       {
*           initialize(2, 44100); // you need to initialize the base class before any operation.   
*       }
*       protected bool onGetData(out short[] data)
*       {
*           //You need to fill data array with some samples
*           
*           return true; //or false if you want to stop playback
*       }
*       
*       protected bool onStart()
*       {
*           return true;   
*       }  
*   }
*   ------------------------
*/
abstract class SoundStream
{
    ~this()
    {
        stop();
        if (m_soundStream !is null)
            sfSoundStream_Destroy(m_soundStream);
        s_instances.remove(m_id);
    }
    
    /**
    *   Start playing the stream
    */        
    void play()
    {
        if (m_soundStream !is null)
        {
            m_flag = true;           
            sfSoundStream_Play(m_soundStream);
            m_t = new Thread(&threadPoll);
            m_t.launch();
        }
    }
    
    /**
    *   Pause the stream
    */        
    void pause()
    {
        if (m_soundStream !is null)
            sfSoundStream_Pause(m_soundStream);
    }

    /**
    *   Stop the stream
    */        
    void stop()
    {
        m_flag = false;
        if (m_soundStream !is null)
            sfSoundStream_Stop(m_soundStream);
        m_t.wait();
        if (m_dummy !is null)
            delete m_dummy;
    }
    
    /**
    *   Get number of channels of the stream
    *   
    *   Returns:
    *       number of channels            
    */        
    uint getChannelsCount()
    {
        return m_channelsCount;
    }

    /**
    *   Get the sample rate of the stream
    *   
    *   Returns:
    *       sample rate            
    */        
    uint getSampleRate()
    {
        return m_sampleRate;
    }

    /**
    *   Get the current status of the stream
    *   
    *   Returns:
    *       Current stream status            
    */        
    SoundStatus getStatus()
    {
        SoundStatus status = sfSoundStream_GetStatus(m_soundStream);
        return status;
    }
    
    /**
    *   Set the sound pitch.
    *   The default pitch is 1
    *
    *   Params:
    *       pitch = New pitch
    */
    void setPitch(float pitch)
    {
        if (m_soundStream !is null)
            sfSoundStream_SetPitch(m_soundStream, pitch);
    }

    /**
    *   Set the sound volume.
    *   The default volume is 100
    *
    *   Params:
    *       volume = Volume (in range [0, 100])
    */
    void setVolume(float volume)
    in
    {
        assert(volume >= 0.f && volume <= 100.f);
    }
    body
    {
        if (m_soundStream !is null)
            sfSoundStream_SetVolume(m_soundStream, volume);
    }

    /*
    *   Set the sound position (take 3 values).
    *   The default position is (0, 0, 0)
    *
    *   Params:
    *      x, y, z = Position of the sound in the world
    */

    void setPosition(float x, float y, float z)
    {
        if (m_soundStream !is null)
            sfSoundStream_SetPosition(m_soundStream, x, y, z);
    }

    /**
    *   Set the sound position (take 3 values).
    *   The default position is (0, 0, 0)
    *
    *   Params:
    *     vec = Position of the sound in the world
    *
    */

    void setPosition(Vector3f vec)
    {
        if (m_soundStream !is null)
            sfSoundStream_SetPosition(m_soundStream, vec.x, vec.y, vec.z);
    }

    /**
    *   Set the minimum distance - closer than this distance,
    *   the listener will hear the sound at its maximum volume.
    *   The default minimum distance is 1.0
    *
    *   Params:
    *       minDistance = New minimum distance for the sound
    */
    void setMinDistance(float minDistance)
    {
        if (m_soundStream !is null)
            sfSoundStream_SetMinDistance(m_soundStream, minDistance);
    }

    /**
    *   Set the attenuation factor - the higher the attenuation, the
    *   more the sound will be attenuated with distance from listener.
    *   The default attenuation factor 1.0
    *
    *   Params: 
    *       attenuation = New attenuation factor for the sound
    *
    */
    void setAttenuation(float attenuation)
    {
        if (m_soundStream !is null)
            sfSoundStream_SetAttenuation(m_soundStream, attenuation);
    }

    /**
    *   Get the pitch
    *
    *   Returns:
    *       Pitch value
    */

    float getPitch()
    {
        if (m_soundStream !is null)
            return sfSoundStream_GetPitch(m_soundStream);
        return 0.f;
    }
    /**
    *   Get the volume
    *
    *   Returns:
    *       Volume value (in range [1, 100])
    *
    */

    float getVolume()
    {
        if (m_soundStream !is null)
            return sfSoundStream_GetVolume(m_soundStream);
        return 0.f;
    }

    /**
    *   Get the sound position
    *   
    *   Returns:
    *       Sound position                
    */        
    Vector3f getPosition()
    {
        Vector3f vec;
        if (m_soundStream !is null)
            sfSoundStream_GetPosition(m_soundStream, &vec.x, &vec.y, &vec.z);
        return vec;
    }

    /**
    *   Get the minimum distance
    *   
    *   Returns:
    *       Get the minimum distance of the sound            
    */        
    float getMinDistance()
    {
        if (m_soundStream !is null)
            return sfSoundStream_GetMinDistance(m_soundStream);
        return 0.f;
    }

    /**
    *   Get the attenuation
    *   
    *   Returns:
    *       Get the attenuation of the sound            
    */        
    float getAttenuation()
    {
        if (m_soundStream !is null)
            return sfSoundStream_GetAttenuation(m_soundStream);
        return 0.f;
    }

protected:
    /**
    *   Protected constructor
    */        
    this()
    {
        m_mutex = new Mutex();
        
        m_samples = new LinkedList!(Data);
        
        m_t = new Thread(&this.threadPoll);
        
        m_id = ++s_seed;
        s_instances[m_id] = this;
    }
    
    /**
    *   Initialize the stream
    *   
    *   Params:
    *       channelsCount = number of channel
    *       sampleRate = sample rate of the stream
    *       
    *   Remarks:
    *       if you don"t call this method before any streaming, object will be in an invalid state.                            
    */        
    void initialize(uint channelsCount, uint sampleRate)
    {
        m_channelsCount = channelsCount;
        m_sampleRate = sampleRate;
        m_soundStream = sfSoundStream_Create(&externalOnStart, &externalOnGetData, channelsCount, sampleRate, &m_id);
    }
    
    /**
    *   Called each time the stream restart
    *   
    *   Returns:
    *       false to abort the playback            
    */        
    abstract bool onStart();
    
    /**
    *   Called each time the stream needs new data.
    *   This method will be call by an other thread, take care of possible synchronisation issues.
    *   
    *   Params:
    *       data = array of samples to stream
    *   
    *   Returns:
    *       true to continue streaming, false to stop                        
    */        
    abstract bool onGetData(out short[] data);
private:

    // Called sync with thread calling play()
    extern(C) static int externalOnStart(void* user)
    {
        int id;
        if ((id = *cast(int*) user) in s_instances)
        {
            SoundStream temp = s_instances[id];
            return (temp.m_flag = temp.onStart());
        }
        return true;
    }
    
    // C Thread callback (no allocation can be done)
    extern (C) static int externalOnGetData(sfSoundStreamChunk* data, void* user)
    {
        int id, flag = false;
        // Get calling soundStream
        if ((id = *cast(int*) user) in s_instances)
        {
            SoundStream temp = s_instances[id];
            //if no samples are available but streaming is not stopped, we sleep the thread
            while (temp.m_samples.empty && temp.m_flag)
                sleep(0.01f);
            
            scope Lock l = new Lock(temp.m_mutex);
            if (!temp.m_samples.empty)
            {
                if (temp.m_dummy !is null)
                    delete temp.m_dummy;
                    
                temp.m_dummy = temp.m_samples.dequeue;
                
                if ((flag = temp.m_dummy.Flag) == true)
                {
                    data.Samples = temp.m_dummy.Samples.ptr;
                    data.NbSamples = temp.m_dummy.Samples.length;
                }
                else
                {
                    data.Samples = null;
                    data.NbSamples = 0;
                }
            }
        }
        return flag;
    }
    
    // Managed thread loop
    void threadPoll(void* dummy)
    {
        short[] data;
        bool ret = true;
        // while streaming is active ...
        while (ret && m_flag)
        {
            {
                scope Lock l = new Lock(m_mutex);
                // see how many samples are available (keep always 2 samples ready)
                if (m_samples.getCount < 2)
                {
                    // if we need new samples, lock and call derived class
                    ret = onGetData(data);
                    m_samples.enqueue(new Data(data, ret));
                }
            }
            sleep(0.1f);
        }
    }
    
    private class Data
    {
        short[] Samples;
        bool Flag;
        
        mixin Alloc;
        
        this (short[] samples, bool flag)
        {
            this.Samples = samples;
            this.Flag = flag;
        }
    }
        
    Thread m_t;
    Mutex m_mutex;
    LinkedList!(Data) m_samples;
    Data m_dummy;
    
    bool m_flag;
    
    sfSoundStream* m_soundStream;
    
    uint m_channelsCount;
    uint m_sampleRate;
    
    int m_id;
    static SoundStream[int] s_instances;
    static int s_seed = 0;
    
}
